#!/usr/bin/env python2.5
# (c) Copyright 2009 Cloudera, Inc.

import os
import sys
import optparse
import manifest
import logging
import textwrap
from git_command import GitCommand
from git_repo import GitRepo

LOADED_MANIFEST = None
def load_manifest():
  global LOADED_MANIFEST
  if not LOADED_MANIFEST:
    LOADED_MANIFEST = manifest.load_manifest("manifest.json")
  return LOADED_MANIFEST

def help(args):
  """Shows help"""
  if len(args) == 1:
    command = args[0]
    doc = COMMANDS[command].__doc__
    if doc:
      print >>sys.stderr, "Help for command %s:\n" % command
      print >>sys.stderr, doc
      sys.exit(1)
  usage()


def init(args):
  """Initializes repository - DEPRECATED - use sync"""
  print >>sys.stderr, "crepo init is deprecated - use crepo sync"
  sync(args)

def sync(args):
  """Synchronize your local repository with the manifest and the real world.
  This includes:
    - ensures that all projects are cloned
    - ensures that they have the correct remotes set up
    - fetches from the remotes
    - checks out the correct tracking branches
    - if the local branch is not dirty and it is a fast-forward update, merges
      the remote branch's changes in

  Process exit code will be 0 if all projects updated correctly.
  """
  
  man = load_manifest()

  for (name, project) in man.projects.iteritems():
    if not project.is_cloned():
      project.clone()
  ensure_remotes([])
  fetch([])
  checkout_branches([])

  retcode = 0
  
  for project in man.projects.itervalues():
    repo = project.git_repo
    if repo.is_workdir_dirty() or repo.is_index_dirty():
      print >>sys.stderr, "Not syncing project %s - it is dirty." % project.name
      retcode = 1
      continue
    (left, right) = project.tracking_status
    if left > 0:
      print >>sys.stderr, \
            ("Not syncing project %s - you have %d unpushed changes." % 
             (project.name, left))
      retcode = 1
      continue
    elif right > 0:
      repo.check_command(["merge", project.remote_ref])
    else:
      print >>sys.stderr, "Project %s needs no update" % project.name

  return retcode

def ensure_remotes(args):
  """Ensure that remotes are set up"""
  man = load_manifest()
  for (proj_name, project) in man.projects.iteritems():
    project.ensure_remotes()


def ensure_tracking_branches(args):
  """Ensures that the tracking branches are set up"""
  man = load_manifest()
  for (name, project) in man.projects.iteritems():
    project.ensure_tracking_branch()

def check_dirty(args):
  """Prints output if any projects have dirty working dirs or indexes."""
  man = load_manifest()
  any_dirty = False
  for (name, project) in man.projects.iteritems():
    repo = project.git_repo
    any_dirty = check_dirty_repo(repo) or any_dirty
  return any_dirty

def check_dirty_repo(repo, indent=0):
  workdir_dirty = repo.is_workdir_dirty()
  index_dirty = repo.is_index_dirty()

  name = repo.name
  if workdir_dirty:
    print " " * indent + "Project %s has a dirty working directory (unstaged changes)." % name
  if index_dirty:
    print " " * indent + "Project %s has a dirty index (staged changes)." % name

  return workdir_dirty or index_dirty


def checkout_branches(args):
  """Checks out the tracking branches listed in the manifest."""

  if check_dirty([]) and '-f' not in args:
    raise Exception("Cannot checkout new branches with dirty projects.")
  
  man = load_manifest()
  for (name, project) in man.projects.iteritems():
    print >>sys.stderr, "Checking out tracking branch in project: %s" % name
    project.checkout_tracking_branch()

def hard_reset_branches(args):
  """Hard-resets your tracking branches to match the remotes."""
  checkout_branches(args)
  man = load_manifest()
  for (name, project) in man.projects.iteritems():
    print >>sys.stderr, "Hard resetting tracking branch in project: %s" % name
    project.git_repo.check_command(["reset", "--hard", project.remote_ref])
  

def do_all_projects(args):
  """Run the given git-command in every project

  Pass -p to do it in parallel"""
  man = load_manifest()

  if args[0] == '-p':
    parallel = True
    del args[0]
  else:
    parallel = False

  towait = []

  for (name, project) in man.projects.iteritems():
    print >>sys.stderr, "In project: ", name, " running ", " ".join(args)
    p = project.git_repo.command_process(args)
    if not parallel:
      p.Wait()
      print >>sys.stderr
    else:
      towait.append(p)

  for p in towait:
    p.Wait()
      
def do_all_projects_remotes(args):
  """Run the given git-command in every project, once for each remote.

  Pass -p to do it in parallel"""
  man = load_manifest()

  if args[0] == '-p':
    parallel = True
    del args[0]
  else:
    parallel = False
  towait = []

  for (name, project) in man.projects.iteritems():
    for remote_name in project.remotes.keys():
      cmd = [arg % {"remote": remote_name} for arg in args]
      print >>sys.stderr, "In project: ", name, " running ", " ".join(cmd)
      p = project.git_repo.command_process(cmd)
      if not parallel:
        p.Wait()
        print >>sys.stderr
      else:
        towait.append(p)
  for p in towait:
    p.Wait()


def fetch(args):
  """Run git-fetch in every project"""
  do_all_projects_remotes(
    args + ["fetch", "-t", "%(remote)s",
            "refs/heads/*:refs/remotes/%(remote)s/*"])

def pull(args):
  """Run git-pull in every project"""
  do_all_projects(args + ["pull", "-t"])

def _format_tracking(local_branch, remote_branch,
                     left, right):
  """
  Takes a tuple returned by repo.tracking_status and outputs a nice string
  describing the state of the repository.
  """
  if (left,right) == (0,0):
    return "Your tracking branch and remote branches are up to date."
  elif left == 0:
    return ("The remote branch %s is %d revisions ahead of tracking branch %s." %
            (remote_branch, right, local_branch))
  elif right == 0:
    return ("Your tracking branch %s is %s revisions ahead of remote branch %s." %
            (local_branch, left, remote_branch))
  else:
    return (("Your local branch %s and remote branch %s have diverged by " +
            "%d and %d revisions.") %
            (local_branch, remote_branch, left, right))

def project_status(project, indent=0):
  repo = project.git_repo
  repo_status(repo, project.tracking_branch, project.remote_ref, indent=indent)

def repo_status(repo, tracking_branch, remote_ref, indent=0):
  # Make sure the right branch is checked out
  if repo.current_branch() != tracking_branch:
    print " " * indent + ("Checked out branch is %s instead of %s" %
                         (repo.current_branch(), tracking_branch))

  # Make sure the branches exist
  has_tracking = repo.has_ref(tracking_branch)
  has_remote = repo.has_ref(remote_ref)

  if not has_tracking:
    print " " * indent + "You appear to be missing the tracking branch " + \
          tracking_branch
  if not has_remote:
    print " " * indent + "You appear to be missing the remote branch " + \
          remote_ref

  if not has_tracking or not has_remote:
    return

  # Print tracking branch status
  (left, right) = repo.tracking_status(tracking_branch, remote_ref)
  text = _format_tracking(tracking_branch, remote_ref, left, right)
  indent_str = " " * indent
  print textwrap.fill(text, initial_indent=indent_str, subsequent_indent=indent_str)

def status(args):
  """Shows where your branches have diverged from the specified remotes."""
  ensure_tracking_branches([])
  man = load_manifest()
  first = True
  for (name, project) in man.projects.iteritems():
    if not first: print
    first = False

    print "Project %s:" % name
    project_status(project, indent=2)
    check_dirty_repo(project.git_repo,
                     indent=2)

  man_repo = get_manifest_repo()
  if man_repo:
    print
    print "Manifest repo:"
    repo_status(man_repo,
                man_repo.current_branch(),
                "origin/" + man_repo.current_branch(),
                indent=2)
    check_dirty_repo(man_repo, indent=2)


def get_manifest_repo():
  """
  Return a GitRepo object pointing to the repository that contains
  the crepo manifest.
  """
  # root dir is cwd for now
  cdup = GitRepo(".").command_process(["rev-parse", "--show-cdup"],
                                      capture_stdout=True,
                                      capture_stderr=True)
  if cdup.Wait() != 0:
    return None
  cdup_path = cdup.stdout.strip()
  if cdup_path:
    return GitRepo(cdup_path)
  else:
    return GitRepo(".")

def dump_refs(args):
  """
  Output a list of all repositories along with their
  checked out branches and their hashes.
  """
  man = load_manifest()
  first = True
  for (name, project) in man.projects.iteritems():
    if not first: print
    first = False
    print "Project %s:" % name

    repo = project.git_repo
    print "  HEAD: %s" % repo.rev_parse("HEAD")
    print "  Symbolic: %s" % repo.current_branch()
    project_status(project, indent=2)

  repo = get_manifest_repo()
  if repo:
    print
    print "Manifest repo:"
    print "  HEAD: %s" % repo.rev_parse("HEAD")
    print "  Symbolic: %s" % repo.current_branch()
    repo_status(repo,
                repo.current_branch(),
                "origin/" + repo.current_branch(),
                indent=2)
    check_dirty_repo(repo, indent=2)
  


COMMANDS = {
  'help': help,
  'init': init,
  'sync': sync,
  'checkout': checkout_branches,
  'hard-reset': hard_reset_branches,
  'do-all': do_all_projects,
  'fetch': fetch,
  'pull': pull,
  'status': status,
  'check-dirty': check_dirty,
  'setup-remotes': ensure_remotes,
  'dump-refs': dump_refs
  }

def usage():
  print >>sys.stderr, "you screwed up. here are the commands:"
  print >>sys.stderr

  max_comlen = 0
  out = []
  for (command, function) in COMMANDS.iteritems():
    docs = function.__doc__ or "no docs"
    docs = docs.split("\n")[0]
    if len(command) > max_comlen:
      max_comlen = len(command)

    out.append( (command, docs) )

  for (command, docs) in sorted(out):
    command += " " * (max_comlen - len(command))
    available_len = 80 - max_comlen - 5
    output_docs = []
    cur_line = ""
    for word in docs.split(" "):
      if cur_line and len(cur_line + " " + word) > available_len:
        output_docs.append(cur_line)
        cur_line = " " * (max_comlen + 5)
      cur_line += " " + word
    if cur_line: output_docs.append(cur_line)
    print >>sys.stderr, "  %s   %s" % (command, "\n".join(output_docs))
  sys.exit(1)

def main():
  args = sys.argv[1:]
  if len(args) == 0 or args[0] not in COMMANDS:
    usage()
  command = COMMANDS[args[0]]
  sys.exit(command.__call__(args[1:]))

if __name__ == "__main__":
  main()



#!/usr/bin/env python2.5
# (c) Copyright 2009 Cloudera, Inc.
import logging
import os
import simplejson

from git_command import GitCommand
from git_repo import GitRepo


class Manifest(object):
  def __init__(self,
               base_dir=None,
               remotes=[],
               projects={},
               default_ref="master",
               default_remote="origin"):
    self.base_dir = base_dir or os.getcwd()
    self.remotes = remotes
    self.projects = projects
    self.default_ref = default_ref
    self.default_remote = default_remote

  @staticmethod
  def from_dict(data, base_dir=None):
    remotes = dict([(name, Remote.from_dict(d)) for (name, d) in data.get('remotes', {}).iteritems()])

    default_remote = data.get("default-remote", "origin")
    assert default_remote in remotes

    man = Manifest(
      base_dir=base_dir,
      default_ref=data.get("default-revision", "master"),
      default_remote=default_remote,
      remotes=remotes)
    
    for (name, d) in data.get('projects', {}).iteritems():
      man.add_project(Project.from_dict(
        manifest=man,
        name=name,
        data=d))
    return man

  @classmethod
  def from_json_file(cls, path):
    data = simplejson.load(file(path))
    return cls.from_dict(data, base_dir=os.path.abspath(os.path.dirname(path)))

  def add_project(self, project):
    if project.name in self.projects:
      raise Exception("Project %s already in manifest" % project.name)
    self.projects[project.name] = project

  def to_json(self):
    return simplejson.dumps(self.data_for_json(), indent=2)

  def data_for_json(self):
    return {
      "default-revision": self.default_ref,
      "default-remote": self.default_remote,
      "remotes": dict( [(name, remote.data_for_json()) for (name, remote) in self.remotes.iteritems()] ),
      "projects": dict( [(name, project.data_for_json()) for (name, project) in self.projects.iteritems()] ),
      }

  def __repr__(self):
    return self.to_json()


class Remote(object):
  def __init__(self,
               fetch):
    self.fetch = fetch

  @staticmethod
  def from_dict(data):
    return Remote(fetch=data.get('fetch'))

  def to_json(self):
    return simplejson.dumps(self.data_for_json(), indent=2)

  def data_for_json(self):
    return {'fetch': self.fetch}

class Project(object):
  def __init__(self,
               name=None,
               manifest=None,
               remotes=None,
               tracking_branch=None, # the name of the tracking branch
               remote_ref="master", # what ref to track
               tracks_remote_ref=True,
               from_remote="origin", # where to pull from
               dir=None,
               remote_project_name = None
               ):
    self.name = name
    self.manifest = manifest
    self.remotes = remotes if remotes else []
    self._dir = dir if dir else name
    self.from_remote = from_remote
    self.remote_ref = remote_ref
    self.tracks_remote_ref = tracks_remote_ref
    self.tracking_branch = tracking_branch
    self.remote_project_name = remote_project_name if remote_project_name else name

  @staticmethod
  def from_dict(manifest, name, data):
    my_remote_names = data.get('remotes', [manifest.default_remote])
    my_remotes = dict([ (r, manifest.remotes[r])
                        for r in my_remote_names])

    from_remote = data.get('from-remote')
    if not from_remote:
      if len(my_remote_names) == 1:
        from_remote = my_remote_names[0]
      elif manifest.default_remote in my_remote_names:
        from_remote = manifest.default_remote
      else:
        raise Exception("no from-remote listed for project %s, and more than one remote" %
                        name)
    
    assert from_remote in my_remote_names
    remote_project_name = data.get('remote-project-name')

    track_tag = data.get('track-tag')
    track_branch = data.get('track-branch')
    track_hash = data.get('track-hash')


    if len([ x for x in [track_tag, track_branch, track_hash] if x]) > 1:
      raise Exception("Cannot specify more than one of track-branch, track-tag, or track-hash for project %s" %
                      name)

    # This is old and deprecated
    ref = data.get('refspec')
    if not track_tag and not track_branch and not track_hash:
      if ref:
        logging.warn("'ref' is deprecated - use either track-branch or track-tag " +
                     "for project %s" % name)
        track_branch = ref
      else:
        track_branch = "master"
    
    if track_tag:
      remote_ref = "refs/tags/" + track_tag
      tracks_remote_ref = True
      tracking_branch = track_tag
    elif track_branch:
      remote_ref = "%s/%s" % (from_remote, track_branch)
      tracks_remote_ref = True
      tracking_branch = track_branch
    elif track_hash:
      remote_ref = track_hash
      tracks_remote_ref = False
      tracking_branch = "crepo"
    else:
      assert False and "Cannot get here!"
      

    return Project(name=name,
                   manifest=manifest,
                   remotes=my_remotes,
                   remote_ref=remote_ref,
                   tracks_remote_ref=tracks_remote_ref,
                   tracking_branch=tracking_branch,
                   dir=data.get('dir', name),
                   from_remote=from_remote,
                   remote_project_name=remote_project_name)


  @property
  def tracking_status(self):
    return self.git_repo.tracking_status(
      self.tracking_branch, self.remote_ref)

  def to_json(self):
    return simplejson.dumps(self.data_for_json())

  def data_for_json(self):
    return {'name': self.name,
            'remotes': self.remotes.keys(),
            'ref': self.ref,
            'from-remote': self.from_remote,
            'dir': self.dir}

  @property
  def dir(self):
    return os.path.join(self.manifest.base_dir, self._dir)

  @property
  def git_repo(self):
    return GitRepo(self.dir)

  def is_cloned(self):
    return self.git_repo.is_cloned()

  ############################################################
  # Actual actions to be taken on a project
  ############################################################

  def clone(self):
    if self.is_cloned():
      return
    logging.warn("Initializing project: %s" % self.name)
    clone_remote = self.manifest.remotes[self.from_remote]
    clone_url = clone_remote.fetch % {"name": self.remote_project_name}
    p = GitCommand(["clone", "-o", self.from_remote, "-n", clone_url, self.dir])
    p.Wait()

    repo = self.git_repo
    if repo.command(["show-ref", "-q", "HEAD"]) != 0:
      # There is no HEAD (maybe origin/master doesnt exist) so check out the tracking
      # branch
      if self.tracks_remote_ref:
        repo.check_command(["checkout", "--track", "-b", self.tracking_branch,
                          self.remote_ref])
      else:
        repo.check_command(["checkout", "-b", self.tracking_branch,
                          self.remote_ref])
    else:
      repo.check_command(["checkout"])
    

  def ensure_remotes(self):
    repo = self.git_repo
    for remote_name in self.remotes:
      remote = self.manifest.remotes[remote_name]
      new_url = remote.fetch % { "name": self.remote_project_name }

      p = repo.command_process(["config", "--get", "remote.%s.url" % remote_name],
                               capture_stdout=True)
      if p.Wait() == 0:
        cur_url = p.stdout.strip()
        if cur_url != new_url:
          repo.check_command(["config", "--replace-all", "remote.%s.url" % remote_name, new_url])
      else:
        repo.check_command(["remote", "add", remote_name, new_url])

  def ensure_tracking_branch(self):
    """Ensure that the tracking branch exists."""
    if not self.is_cloned():
      self.init()

    branch_missing = self.git_repo.command(
      ["rev-parse", "--verify", "-q", "refs/heads/%s" % self.tracking_branch],
      capture_stdout=True)

    if branch_missing:
      logging.warn("Branch %s does not exist in project %s. checking out." %
                   (self.tracking_branch, self.name))
      if self.tracks_remote_ref:
        self.git_repo.command(["branch", "--track",
                      self.tracking_branch, self.remote_ref])
      else:
        self.git_repo.command(["branch", self.tracking_branch, self.remote_ref])
    

  def checkout_tracking_branch(self):
    """Check out the correct tracking branch."""
    self.ensure_tracking_branch()
    self.git_repo.check_command(["checkout", self.tracking_branch])

def load_manifest(path):
  return Manifest.from_json_file(path)


def test_json_load_store():
  man = load_manifest(os.path.join(os.path.dirname(__file__), 'test', 'test_manifest.json'))
  assert len(man.to_json()) > 10



#
# Copyright (C) 2008 The Android Open Source Project
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import subprocess
from trace import REPO_TRACE, IsTrace, Trace

GIT = 'git'
MIN_GIT_VERSION = (1, 5, 4)
GIT_DIR = 'GIT_DIR'


LAST_GITDIR = None
LAST_CWD = None

class GitCommand(object):
  def __init__(self,
               cmdv,
               capture_stdout = False,
               capture_stderr = False,
               cwd = None):
    env = dict(os.environ)

    for e in [REPO_TRACE,
              GIT_DIR,
              'GIT_ALTERNATE_OBJECT_DIRECTORIES',
              'GIT_OBJECT_DIRECTORY',
              'GIT_WORK_TREE',
              'GIT_GRAFT_FILE',
              'GIT_INDEX_FILE']:
      if e in env:
        del env[e]

    command = [GIT]
    command.extend(cmdv)

    stdout = subprocess.PIPE if capture_stdout else None
    stderr = subprocess.PIPE if capture_stderr else None

    if IsTrace():
      global LAST_CWD
      global LAST_GITDIR

      dbg = ''

      if cwd and LAST_CWD != cwd:
        if LAST_GITDIR or LAST_CWD:
          dbg += '\n'
        dbg += ': cd %s\n' % cwd
        LAST_CWD = cwd

      if GIT_DIR in env and LAST_GITDIR != env[GIT_DIR]:
        if LAST_GITDIR or LAST_CWD:
          dbg += '\n'
        dbg += ': export GIT_DIR=%s\n' % env[GIT_DIR]
        LAST_GITDIR = env[GIT_DIR]

      dbg += ': '
      dbg += ' '.join(command)
      if stdout == subprocess.PIPE:
        dbg += ' 1>|'
      if stderr == subprocess.PIPE:
        dbg += ' 2>|'
      Trace('%s', dbg)

    try:
      p = subprocess.Popen(command,
                           cwd = cwd,
                           env = env,
                           stdout = stdout,
                           stderr = stderr)
    except Exception, e:
      raise Exception('%s: %s' % (command[1], e))

    self.process = p
    self.stdin = p.stdin

  def Wait(self):
    p = self.process

    if p.stdin:
      p.stdin.close()
      self.stdin = None

    if p.stdout:
      self.stdout = p.stdout.read()
      p.stdout.close()
    else:
      p.stdout = None

    if p.stderr:
      self.stderr = p.stderr.read()
      p.stderr.close()
    else:
      p.stderr = None

    return self.process.wait()



#
# Copyright (C) 2008 The Android Open Source Project
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
import os
REPO_TRACE = 'REPO_TRACE'

try:
  _TRACE = os.environ[REPO_TRACE] == '1'
except KeyError:
  _TRACE = False

def IsTrace():
  return _TRACE

def SetTrace():
  global _TRACE
  _TRACE = True

def Trace(fmt, *args):
  if IsTrace():
    print >>sys.stderr, fmt % args



#!/usr/bin/env python2.5
# (c) Copyright 2009 Cloudera, Inc.

from git_command import GitCommand
import os

class GitRepo(object):
  def __init__(self, path):
    self.path = path

  def command(self, cmdv, **kwargs):
    """
    Runs the given git command and returns the status code returned.

    @param cmdv the git command as a list of args (eg ["diff"])
    """
    return self.command_process(cmdv, **kwargs).Wait()
  
  def check_command(self, cmdv, capture_stdout=False):
    """
    Runs the given git command, and raises an Exception if the status
    code returned is non-zero.
    """
    p = self.command_process(cmdv, capture_stdout=capture_stdout)
    rc = p.Wait()
    if rc != 0:
      raise Exception("Command %s returned non-zero exit code: %d" %
                      (repr(cmdv), rc))
    if capture_stdout:
      return p.stdout

  def command_process(self, cmdv, **kwargs):
    p = GitCommand(cwd=self.path,
                   cmdv=cmdv,
                   **kwargs)
    return p

  def is_cloned(self):
    return os.path.exists(os.path.join(self.path, ".git"))

  def is_dirty(self):
    return self.is_workdir_dirty() or self.is_index_dirty()

  def is_workdir_dirty(self):
    return self.command(["diff", "--quiet"])

  def is_index_dirty(self):
    return self.command(["diff", "--quiet", "--cached"])

  def tracking_status(self, local_branch, remote_branch):
    """
    Return a tuple (left_commits, right_commits). The first element
    is the number of commits in the local branch and not in remote.
    The second element is the other direction
    """
    stdout = self.check_command(["rev-list", "--left-right",
                                 "%s...%s" % (local_branch, remote_branch)],
                                capture_stdout=True)
    commits = stdout.strip().split("\n")
    left_commits, right_commits = (0,0)
    for commit in commits:
      if not commit: continue
      if commit[0] == '<':
        left_commits += 1
      else:
        right_commits += 1

    return (left_commits, right_commits)

  def current_branch(self):
    stdout = self.check_command(["symbolic-ref", "HEAD"],
                                capture_stdout=True)
    return stdout.rstrip().replace("refs/heads/", "")


  def rev_parse(self, rev):
    stdout = self.check_command(['rev-parse', 'HEAD'],
                                capture_stdout=True)
    return stdout.rstrip()

  def has_ref(self, ref):
    p = self.command_process(['rev-parse', '--verify', '-q', ref],
                             capture_stdout=True)
    return p.Wait() == 0
    

  @property
  def name(self):
    return os.path.basename(os.path.realpath(self.path))



#!/usr/bin/env python

from setuptools import setup, find_packages

setup(
    name="crepo",
    version='0.1',
    url="http://github.com/cloudera/crepo/tree/master",
    maintainer="Chad Metcalf",
    maintainer_email="chad@cloudera.com",
    py_modules=['crepo', 'git_command', 'git_repo', 'manifest', 'trace', 'test' ],
    entry_points = { 'console_scripts': [ 'crepo = crepo:main', ], },
    data_files=[('shell-tests', ['shell-tests/basic_test.sh', 'shell-tests/follow_tag.sh', 'shell-tests/smoke_test.sh']),],
    install_requires = ['setuptools', 'simplejson'],
)



#!/usr/bin/env python2.5
# (c) Copyright 2009 Cloudera, Inc.

import unittest
from unittest import TestCase
import os
import subprocess
import re

TESTS_DIR=os.path.join(os.path.dirname(__file__), "shell-tests")

class ShellTests(TestCase):
  def _run_shell_test(self, path):
    print "running: " + path
    ret = subprocess.call([os.path.join(TESTS_DIR, path)])
    self.assertEquals(ret, 0)

def __add_tests():
  for x in os.listdir(TESTS_DIR):
    if x.startswith("."): continue
    t = lambda self,x=x: self._run_shell_test(x)
    t.__name__ = 'test' + re.sub('[^a-zA-Z0-9]', '_', 'test' + x)
    setattr(ShellTests, t.__name__,  t)
__add_tests()

if __name__ == "__main__":
  unittest.main()
